==> klayout-0.14.tar.gz: klayout-0.14/src/gsiDeclDbPath.cc <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "gsi.h"
#include "dbPoint.h"
#include "dbPath.h"

namespace gsi
{

// ---------------------------------------------------------------
//  path binding

template <class C>
struct path_defs
{
  typedef typename C::coord_type coord_type;
  typedef typename C::box_type box_type;
  typedef typename C::point_type point_type;
  typedef typename C::distance_type distance_type;
  typedef typename C::area_type area_type;
  typedef db::simple_trans<coord_type> simple_trans_type;
  typedef db::complex_trans<coord_type, double> complex_trans_type;

  static void set_points (C *c, const std::vector<point_type> &pts)
  {
    return c->assign (pts.begin (), pts.end ());
  }

  static C *new_v ()
  {
    return new C ();
  }

  static C *new_pw (const std::vector<point_type> &pts, coord_type width)
  {
    return new C (pts.begin (), pts.end (), width);
  }

  static C *new_pwx (const std::vector<point_type> &pts, coord_type width, coord_type bgn_ext, coord_type end_ext)
  {
    return new C (pts.begin (), pts.end (), width, bgn_ext, end_ext);
  }

  static C *new_pwxr (const std::vector<point_type> &pts, coord_type width, coord_type bgn_ext, coord_type end_ext, bool round)
  {
    return new C (pts.begin (), pts.end (), width, bgn_ext, end_ext, round);
  }

  static gsi::Methods methods ()
  {
    return
    constructor ("new", &new_v, 
      "@brief Default constructor: creates an empty (invalid) path with width 0"
    ) +
    constructor ("new|#new_pw", &new_pw, 
      "@brief Constructor given the points of the path's spine and the width\n"
      "\n"
      "@args pts, width\n"
      "\n"
      "@param pts The points forming the spine of the path\n"
      "@param width The width of the path\n"
    ) +
    constructor ("new|#new_pwx", &new_pwx, 
      "@brief Constructor given the points of the path's spine, the width and the extensions\n"
      "\n"
      "@args pts, width, bgn_ext, end_ext\n"
      "\n"
      "@param pts The points forming the spine of the path\n"
      "@param width The width of the path\n"
      "@param bgn_ext The begin extension of the path\n"
      "@param end_ext The end extension of the path\n"
    ) +
    constructor ("new|#new_pwxr", &new_pwxr, 
      "@brief Constructor given the points of the path's spine, the width, the extensions and the round end flag\n"
      "\n"
      "@args pts, width, bgn_ext, end_ext, round\n"
      "\n"
      "@param pts The points forming the spine of the path\n"
      "@param width The width of the path\n"
      "@param bgn_ext The begin extension of the path\n"
      "@param end_ext The end extension of the path\n"
      "@param round If this flag is true, the path will get rounded ends\n"
    ) +
    method ("<", &C::operator<,
      "@brief Less operator\n"
      "@args p\n"
      "@param p The object to compare against\n"
      "This operator is provided to establish some, not necessarily a certain sorting order"
    ) +
    method ("==", &C::operator==,
      "@brief Equality test\n"
      "@args p\n"
      "@param p The object to compare against"
    ) +
    method ("!=", &C::operator!=,
      "@brief Inequality test\n"
      "@args p\n"
      "@param p The object to compare against\n"
    ) +
    method_ext ("points=", &set_points,
      "@brief Set the points of the path\n"
      "@args p\n"
      "@param p An array of points to assign to the path's spine"
    ) +
    iterator ("each_point", &C::begin, &C::end, 
      "@brief Get the points that make up the path's spine"
    ) +
    method ("points", &C::points,
      "@brief Get the number of points"
    ) +
    method ("width=", (void (C::*)(coord_type)) &C::width,
      "@brief Set the width\n"
    ) +
    method ("width", (coord_type (C::*)() const) &C::width,
      "@brief Get the width\n"
    ) +
    method ("bgn_ext=", (void (C::*)(coord_type)) &C::bgn_ext,
      "@brief Set the begin extension\n"
    ) +
    method ("bgn_ext", (coord_type (C::*)() const) &C::bgn_ext,
      "@brief Get the begin extension\n"
    ) +
    method ("end_ext=", (void (C::*)(coord_type)) &C::end_ext,
      "@brief Set the end extension\n"
    ) +
    method ("end_ext", (coord_type (C::*)() const) &C::end_ext,
      "@brief Get the end extension\n"
    ) +
    method ("round=", (void (C::*)(bool)) &C::round,
      "@brief Set the 'round ends' flag\n"
    ) +
    method ("is_round?", (bool (C::*)() const) &C::round,
      "@brief Tell, if the path has round ends\n"
    ) +
    method ("move", &C::move,
      "@brief Moves the path.\n"
      "@args p\n"
      "\n"
      "Moves the path by the given offset and returns the \n"
      "moved path. The path is overwritten.\n"
      "\n"
      "@param p The distance to move the path.\n"
      "\n"
      "@return The moved path.\n"
    ) +
    method ("moved", &C::moved,
      "@brief Returns the moved path\n"
      "@args p\n"
      "\n"
      "Moves the path by the given offset and returns the \n"
      "moved path. The path is not modified.\n"
      "\n"
      "@param p The distance to move the path.\n"
      "\n"
      "@return The moved path.\n"
    ) +
    method ("transformed", &C::template transformed<simple_trans_type>,
      "@brief Transform the path.\n"
      "@args t\n"
      "\n"
      "Transforms the path with the given transformation.\n"
      "Does not modify the path but returns the transformed path.\n"
      "\n"
      "@param t The transformation to apply.\n"
      "\n"
      "@return The transformed path.\n"
    ) +
    method ("transformed_cplx", &C::template transformed<complex_trans_type>,
      "@brief Transform the path.\n"
      "@args t\n"
      "\n"
      "Transforms the path with the given complex transformation.\n"
      "Does not modify the path but returns the transformed path.\n"
      "\n"
      "@param t The transformation to apply.\n"
      "\n"
      "@return The transformed path.\n"
    ) +
    method ("to_s", (std::string (C::*) () const) &C::to_string,
      "@brief Convert to a string\n"
    ) +
    method ("simple_polygon", &C::simple_polygon,
      "@brief Convert the path to a simple polygon\n"
      "The returned polygon is not guaranteed to be non-selfoverlapping. This may happen if the path overlaps "
      "itself or contains very short segments."
    ) +
    method ("polygon", &C::polygon,
      "@brief Convert the path to a polygon\n"
      "The returned polygon is not guaranteed to be non-selfoverlapping. This may happen if the path overlaps "
      "itself or contains very short segments."
    ) +
    method ("bbox", &C::box,
      "@brief Return the bounding box of the path"
    );
  }
};

template <>
Class<db::Path> Class<db::Path>::decl ("Path", 
  path_defs<db::Path>::methods (),
  "@brief An path class\n"
  "\n"
  "A path consists of an sequence of line segments forming the 'spine' of the path "
  "and a width. In addition, the starting point can be drawn back by a certain extent (the 'begin extension') "
  "and the end point can be pulled forward somewhat (by the 'end extension'). A path may have round ends "
  "for special purposes. "
);

template <>
Class<db::DPath> Class<db::DPath>::decl ("DPath", 
  path_defs<db::DPath>::methods (),
  "@brief An path class\n"
  "\n"
  "A path consists of an sequence of line segments forming the 'spine' of the path "
  "and a width. In addition, the starting point can be drawn back by a certain extent (the 'begin extension') "
  "and the end point can be pulled forward somewhat (by the 'end extension'). A path may have round ends "
  "for special purposes. "
);

}

==> klayout-0.15.tar.gz: klayout-0.15/src/gsiDeclDbPath.cc <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "gsi.h"
#include "dbPoint.h"
#include "dbPath.h"

namespace gsi
{

// ---------------------------------------------------------------
//  path binding

template <class C>
struct path_defs
{
  typedef typename C::coord_type coord_type;
  typedef typename C::box_type box_type;
  typedef typename C::point_type point_type;
  typedef typename C::distance_type distance_type;
  typedef typename C::area_type area_type;
  typedef db::simple_trans<coord_type> simple_trans_type;
  typedef db::complex_trans<coord_type, double> complex_trans_type;

  static void set_points (C *c, const std::vector<point_type> &pts)
  {
    return c->assign (pts.begin (), pts.end ());
  }

  static C *new_v ()
  {
    return new C ();
  }

  static C *new_pw (const std::vector<point_type> &pts, coord_type width)
  {
    return new C (pts.begin (), pts.end (), width);
  }

  static C *new_pwx (const std::vector<point_type> &pts, coord_type width, coord_type bgn_ext, coord_type end_ext)
  {
    return new C (pts.begin (), pts.end (), width, bgn_ext, end_ext);
  }

  static C *new_pwxr (const std::vector<point_type> &pts, coord_type width, coord_type bgn_ext, coord_type end_ext, bool round)
  {
    return new C (pts.begin (), pts.end (), width, bgn_ext, end_ext, round);
  }

  static gsi::Methods methods ()
  {
    return
    constructor ("new", &new_v, 
      "@brief Default constructor: creates an empty (invalid) path with width 0"
    ) +
    constructor ("new|#new_pw", &new_pw, 
      "@brief Constructor given the points of the path's spine and the width\n"
      "\n"
      "@args pts, width\n"
      "\n"
      "@param pts The points forming the spine of the path\n"
      "@param width The width of the path\n"
    ) +
    constructor ("new|#new_pwx", &new_pwx, 
      "@brief Constructor given the points of the path's spine, the width and the extensions\n"
      "\n"
      "@args pts, width, bgn_ext, end_ext\n"
      "\n"
      "@param pts The points forming the spine of the path\n"
      "@param width The width of the path\n"
      "@param bgn_ext The begin extension of the path\n"
      "@param end_ext The end extension of the path\n"
    ) +
    constructor ("new|#new_pwxr", &new_pwxr, 
      "@brief Constructor given the points of the path's spine, the width, the extensions and the round end flag\n"
      "\n"
      "@args pts, width, bgn_ext, end_ext, round\n"
      "\n"
      "@param pts The points forming the spine of the path\n"
      "@param width The width of the path\n"
      "@param bgn_ext The begin extension of the path\n"
      "@param end_ext The end extension of the path\n"
      "@param round If this flag is true, the path will get rounded ends\n"
    ) +
    method ("<", &C::operator<,
      "@brief Less operator\n"
      "@args p\n"
      "@param p The object to compare against\n"
      "This operator is provided to establish some, not necessarily a certain sorting order"
    ) +
    method ("==", &C::operator==,
      "@brief Equality test\n"
      "@args p\n"
      "@param p The object to compare against"
    ) +
    method ("!=", &C::operator!=,
      "@brief Inequality test\n"
      "@args p\n"
      "@param p The object to compare against\n"
    ) +
    method_ext ("points=", &set_points,
      "@brief Set the points of the path\n"
      "@args p\n"
      "@param p An array of points to assign to the path's spine"
    ) +
    iterator ("each_point", &C::begin, &C::end, 
      "@brief Get the points that make up the path's spine"
    ) +
    method ("points", &C::points,
      "@brief Get the number of points"
    ) +
    method ("width=", (void (C::*)(coord_type)) &C::width,
      "@brief Set the width\n"
    ) +
    method ("width", (coord_type (C::*)() const) &C::width,
      "@brief Get the width\n"
    ) +
    method ("bgn_ext=", (void (C::*)(coord_type)) &C::bgn_ext,
      "@brief Set the begin extension\n"
    ) +
    method ("bgn_ext", (coord_type (C::*)() const) &C::bgn_ext,
      "@brief Get the begin extension\n"
    ) +
    method ("end_ext=", (void (C::*)(coord_type)) &C::end_ext,
      "@brief Set the end extension\n"
    ) +
    method ("end_ext", (coord_type (C::*)() const) &C::end_ext,
      "@brief Get the end extension\n"
    ) +
    method ("round=", (void (C::*)(bool)) &C::round,
      "@brief Set the 'round ends' flag\n"
    ) +
    method ("is_round?", (bool (C::*)() const) &C::round,
      "@brief Tell, if the path has round ends\n"
    ) +
    method ("move", &C::move,
      "@brief Moves the path.\n"
      "@args p\n"
      "\n"
      "Moves the path by the given offset and returns the \n"
      "moved path. The path is overwritten.\n"
      "\n"
      "@param p The distance to move the path.\n"
      "\n"
      "@return The moved path.\n"
    ) +
    method ("moved", &C::moved,
      "@brief Returns the moved path\n"
      "@args p\n"
      "\n"
      "Moves the path by the given offset and returns the \n"
      "moved path. The path is not modified.\n"
      "\n"
      "@param p The distance to move the path.\n"
      "\n"
      "@return The moved path.\n"
    ) +
    method ("transformed", &C::template transformed<simple_trans_type>,
      "@brief Transform the path.\n"
      "@args t\n"
      "\n"
      "Transforms the path with the given transformation.\n"
      "Does not modify the path but returns the transformed path.\n"
      "\n"
      "@param t The transformation to apply.\n"
      "\n"
      "@return The transformed path.\n"
    ) +
    method ("transformed_cplx", &C::template transformed<complex_trans_type>,
      "@brief Transform the path.\n"
      "@args t\n"
      "\n"
      "Transforms the path with the given complex transformation.\n"
      "Does not modify the path but returns the transformed path.\n"
      "\n"
      "@param t The transformation to apply.\n"
      "\n"
      "@return The transformed path.\n"
    ) +
    method ("to_s", (std::string (C::*) () const) &C::to_string,
      "@brief Convert to a string\n"
    ) +
    method ("simple_polygon", &C::simple_polygon,
      "@brief Convert the path to a simple polygon\n"
      "The returned polygon is not guaranteed to be non-selfoverlapping. This may happen if the path overlaps "
      "itself or contains very short segments."
    ) +
    method ("polygon", &C::polygon,
      "@brief Convert the path to a polygon\n"
      "The returned polygon is not guaranteed to be non-selfoverlapping. This may happen if the path overlaps "
      "itself or contains very short segments."
    ) +
    method ("bbox", &C::box,
      "@brief Return the bounding box of the path"
    );
  }
};

db::Path from_dp (const db::DPath &d)
{
  return db::Path (db::Path::from_double (d));
}

db::DPath from_p (const db::Path &d)
{
  return db::DPath (d, db::cast_op<db::DPoint, db::Point> (), db::cast_op<double, db::Coord> ());
}

template <>
Class<db::Path> Class<db::Path>::decl ("Path", 
  path_defs<db::Path>::methods () +
  method ("from_dpath", &from_dp, 
    "@brief Construct an integer-coordinate path from a floating-point coordinate one\n"
    "\n"
    "@args double_path\n"
    "\n"
    "This method has been added in version 0.15."
  ),
  "@brief An path class\n"
  "\n"
  "A path consists of an sequence of line segments forming the 'spine' of the path "
  "and a width. In addition, the starting point can be drawn back by a certain extent (the 'begin extension') "
  "and the end point can be pulled forward somewhat (by the 'end extension'). A path may have round ends "
  "for special purposes. "
);

template <>
Class<db::DPath> Class<db::DPath>::decl ("DPath", 
  path_defs<db::DPath>::methods () +
  method ("from_ipath", &from_p, 
    "@brief Construct a floating-point coordinate path from an integer coordinate one"
    "\n"
    "@args int_path\n"
    "\n"
    "This method has been added in version 0.15."
  ),
  "@brief An path class\n"
  "\n"
  "A path consists of an sequence of line segments forming the 'spine' of the path "
  "and a width. In addition, the starting point can be drawn back by a certain extent (the 'begin extension') "
  "and the end point can be pulled forward somewhat (by the 'end extension'). A path may have round ends "
  "for special purposes. "
);

}
